from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from .models import Rest


class RestAdminForm(forms.ModelForm):
    class Meta:
        model = Rest
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super(RestAdminForm, self).__init__(*args, **kwargs)
        self.initial_status = self.instance.status

    def clean(self):
        cleaned_data = super().clean()

        if 'status' in cleaned_data:
            # Check if status has been changed and the user is not a superuser
            if 'status' in self.changed_data and not self.current_user.is_superuser:
                raise ValidationError(_('Only admin users can change the status.'))

        return cleaned_data



"""
    xnbtd
"""

from importlib.metadata import version


__version__ = version('xnbtd')
__author__ = 'André Théo LAURET <andrelauret@eclipse-technology.eu>'


default_app_config = 'xnbtd.apps.XnbtdConfig'



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.admin import GenericTabularInline
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import gettext_lazy as _

from .models import GLS, TNT, BreakTime, ChronopostDelivery, ChronopostPickup, Ciblex


class BreakTimeInline(GenericTabularInline):
    model = BreakTime
    ct_field = "content_type"
    ct_fk_field = "object_id"
    extra = 1
    fields = ["start_time", "end_time"]


class BaseAdmin(admin.ModelAdmin):
    inlines = [BreakTimeInline]
    change_list_template = "xnbtd/admin/change_list.html"

    def display_breaks(self, obj):
        breaks = BreakTime.objects.filter(
            content_type=ContentType.objects.get_for_model(obj), object_id=obj.id
        )
        return ", ".join(
            [f"{b.start_time.strftime('%H:%M')} - {b.end_time.strftime('%H:%M')}" for b in breaks]
        )

    display_breaks.short_description = _("Breaks")

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super().get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super().get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["list_statistic"] = self.list_statistic
        return super().changelist_view(request, extra_context=extra_context)


class GLSAdmin(BaseAdmin):
    list_display = (
        "name",
        "linked_user",
        "date",
        "display_breaks",
        "license_plate",
        "points_charges",
        "points_delivered",
        "packages_charges",
        "packages_delivered",
        "avp_relay",
        "shd",
        "eo",
        "pickup_point",
        "beginning_hour",
        "ending_hour",
        "comments",
    )
    list_filter = ("date", "linked_user", "name", "license_plate")
    list_statistic = [
        ("packages_delivered", _("Total Packages Delivered")),
    ]

    def get_queryset(self, request):
        qs = super(GLSAdmin, self).get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super(GLSAdmin, self).get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super(GLSAdmin, self).get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super(GLSAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["list_statistic"] = self.list_statistic
        return super().changelist_view(request, extra_context=extra_context)

    change_list_template = "xnbtd/admin/change_list.html"


class TNTAdmin(BaseAdmin):
    list_display = (
        "name",
        "linked_user",
        "date",
        "display_breaks",
        "license_plate",
        "client_numbers",
        "refused",
        "avp",
        "cad",
        "totals_clients",
        "occasional_abductions",
        "regular_abductions",
        "totals_clients_abductions",
        "kilometers",
        "beginning_hour",
        "ending_hour",
        "comments",
    )
    list_filter = ("date", "linked_user", "name", "license_plate")
    list_statistic = [
        ("totals_clients", _("Total Clients")),
    ]

    def get_queryset(self, request):
        qs = super(TNTAdmin, self).get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super(TNTAdmin, self).get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super(TNTAdmin, self).get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super(TNTAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["list_statistic"] = self.list_statistic
        return super().changelist_view(request, extra_context=extra_context)

    change_list_template = "xnbtd/admin/change_list.html"


class ChronopostDeliveryAdmin(BaseAdmin):
    list_display = (
        "name",
        "linked_user",
        "date",
        "display_breaks",
        "license_plate",
        "charged_packages",
        "charged_points",
        "including_ip",
        "relay",
        "return_packages",
        "return_points",
        "overdue",
        "anomalies",
        "total_points",
        "full_km",
        "beginning_hour",
        "ending_hour",
        "comments",
    )
    list_filter = ("date", "linked_user", "name", "license_plate")
    list_statistic = [
        ("total_points", _("Total of Points")),
    ]

    def get_queryset(self, request):
        qs = super(ChronopostDeliveryAdmin, self).get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super(ChronopostDeliveryAdmin, self).get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super(ChronopostDeliveryAdmin, self).get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super(ChronopostDeliveryAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs
        )

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["list_statistic"] = self.list_statistic
        return super().changelist_view(request, extra_context=extra_context)

    change_list_template = "xnbtd/admin/change_list.html"


class ChronopostPickupAdmin(BaseAdmin):
    list_display = (
        "name",
        "linked_user",
        "date",
        "display_breaks",
        "license_plate",
        "esd",
        "picked_points",
        "poste",
        "beginning_hour",
        "ending_hour",
        "comments",
    )
    list_filter = ("date", "linked_user", "name", "license_plate")
    list_statistic = [
        ("picked_points", _("Total of Picked Points")),
    ]

    def get_queryset(self, request):
        qs = super(ChronopostPickupAdmin, self).get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super(ChronopostPickupAdmin, self).get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super(ChronopostPickupAdmin, self).get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super(ChronopostPickupAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs
        )

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["list_statistic"] = self.list_statistic
        return super().changelist_view(request, extra_context=extra_context)

    change_list_template = "xnbtd/admin/change_list.html"


class CiblexAdmin(BaseAdmin):
    list_display = (
        "name",
        "linked_user",
        "date",
        "display_breaks",
        "license_plate",
        "type",
        "nights",
        "days",
        "avp",
        "spare_part",
        "synchro",
        "morning_pickup",
        "beginning_hour",
        "ending_hour",
        "comments",
    )
    list_filter = ("date", "linked_user", "name", "license_plate")
    list_statistic = [("days", _("Total Days"))]

    def get_queryset(self, request):
        qs = super(CiblexAdmin, self).get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super(CiblexAdmin, self).get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super(CiblexAdmin, self).get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super(CiblexAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["list_statistic"] = self.list_statistic
        return super().changelist_view(request, extra_context=extra_context)

    change_list_template = "xnbtd/admin/change_list.html"


admin.site.register(GLS, GLSAdmin)
admin.site.register(TNT, TNTAdmin)
admin.site.register(ChronopostDelivery, ChronopostDeliveryAdmin)
admin.site.register(ChronopostPickup, ChronopostPickupAdmin)
admin.site.register(Ciblex, CiblexAdmin)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class PlanningsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'xnbtd.plannings'
    verbose_name = _('app_plannings_name')



from django.apps import AppConfig
from django.contrib.admin.apps import AdminConfig


class XnbtdConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'xnbtd'


class XnbtdAdminConfig(AdminConfig):
    default_site = 'xnbtd.admin.AppAdminSite'



# Generated by Django 4.1.10 on 2023-08-02 18:13

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="TNT",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Tour Name")),
                ("date", models.DateField(verbose_name="Tour Date")),
                ("breaks", models.TimeField(verbose_name="Breaks")),
                ("beginning_hour", models.TimeField(verbose_name="Beginning Hour")),
                ("ending_hour", models.TimeField(verbose_name="Ending Hour")),
                (
                    "total_hour",
                    models.FloatField(blank=True, null=True, verbose_name="Total Hour"),
                ),
                ("client_numbers", models.IntegerField(verbose_name="Client numbers")),
                ("refused", models.IntegerField(verbose_name="Refused")),
                ("avp", models.IntegerField(verbose_name="avp")),
                ("cad", models.IntegerField(verbose_name="cad/return")),
                ("totals_clients", models.IntegerField(verbose_name="totals clients")),
                (
                    "occasional_abductions",
                    models.IntegerField(verbose_name="occasional abductions"),
                ),
                (
                    "regular_abductions",
                    models.IntegerField(verbose_name="regular abductions"),
                ),
                (
                    "totals_clients_abductions",
                    models.IntegerField(verbose_name="total clients abductions"),
                ),
                ("kilometers", models.IntegerField(verbose_name="KM/Full")),
                (
                    "linked_user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Deliveryman",
                    ),
                ),
            ],
            options={
                "verbose_name": "TNT",
                "verbose_name_plural": "TNT",
            },
        ),
        migrations.CreateModel(
            name="GLS",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Tour Name")),
                ("date", models.DateField(verbose_name="Tour Date")),
                ("breaks", models.TimeField(verbose_name="Breaks")),
                ("beginning_hour", models.TimeField(verbose_name="Beginning Hour")),
                ("ending_hour", models.TimeField(verbose_name="Ending Hour")),
                (
                    "total_hour",
                    models.FloatField(blank=True, null=True, verbose_name="Total Hour"),
                ),
                ("points_charges", models.IntegerField(verbose_name="Points Charges")),
                (
                    "points_delivered",
                    models.IntegerField(verbose_name="Points Delivered"),
                ),
                (
                    "packages_charges",
                    models.IntegerField(verbose_name="Packages Charges"),
                ),
                (
                    "packages_delivered",
                    models.IntegerField(verbose_name="Packages Delivered"),
                ),
                ("avp_relay", models.IntegerField(verbose_name="AVP Relay")),
                ("shd", models.IntegerField(verbose_name="SHD")),
                ("eo", models.IntegerField(verbose_name="EO")),
                ("pickup_point", models.IntegerField(verbose_name="Pickup Point")),
                (
                    "linked_user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Deliveryman",
                    ),
                ),
            ],
            options={
                "verbose_name": "GLS",
                "verbose_name_plural": "GLS",
            },
        ),
        migrations.CreateModel(
            name="Ciblex",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Tour Name")),
                ("date", models.DateField(verbose_name="Tour Date")),
                ("breaks", models.TimeField(verbose_name="Breaks")),
                ("beginning_hour", models.TimeField(verbose_name="Beginning Hour")),
                ("ending_hour", models.TimeField(verbose_name="Ending Hour")),
                (
                    "total_hour",
                    models.FloatField(blank=True, null=True, verbose_name="Total Hour"),
                ),
                ("type", models.CharField(default="LIVRAISONS", max_length=255)),
                ("nights", models.IntegerField(verbose_name="nights")),
                ("days", models.IntegerField(verbose_name="days")),
                ("avp", models.IntegerField(verbose_name="avp")),
                ("spare_part", models.IntegerField(verbose_name="spare part")),
                ("synchro", models.IntegerField(verbose_name="synchro")),
                ("morning_pickup", models.IntegerField(verbose_name="morning pickups")),
                (
                    "linked_user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Deliveryman",
                    ),
                ),
            ],
            options={
                "verbose_name": "Ciblex",
                "verbose_name_plural": "Ciblex",
            },
        ),
        migrations.CreateModel(
            name="ChronopostPickup",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Tour Name")),
                ("date", models.DateField(verbose_name="Tour Date")),
                ("breaks", models.TimeField(verbose_name="Breaks")),
                ("beginning_hour", models.TimeField(verbose_name="Beginning Hour")),
                ("ending_hour", models.TimeField(verbose_name="Ending Hour")),
                (
                    "total_hour",
                    models.FloatField(blank=True, null=True, verbose_name="Total Hour"),
                ),
                ("esd", models.IntegerField(verbose_name="ESD")),
                ("picked_points", models.IntegerField(verbose_name="Picked Points")),
                ("poste", models.IntegerField(verbose_name="Poste")),
                (
                    "linked_user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Deliveryman",
                    ),
                ),
            ],
            options={
                "verbose_name": "Chronopost Pickup",
                "verbose_name_plural": "Chronopost Pickup",
            },
        ),
        migrations.CreateModel(
            name="ChronopostDelivery",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Tour Name")),
                ("date", models.DateField(verbose_name="Tour Date")),
                ("breaks", models.TimeField(verbose_name="Breaks")),
                ("beginning_hour", models.TimeField(verbose_name="Beginning Hour")),
                ("ending_hour", models.TimeField(verbose_name="Ending Hour")),
                (
                    "total_hour",
                    models.FloatField(blank=True, null=True, verbose_name="Total Hour"),
                ),
                (
                    "charged_packages",
                    models.IntegerField(verbose_name="Charged Packages"),
                ),
                ("charged_points", models.IntegerField(verbose_name="Charged Points")),
                ("including_ip", models.IntegerField(verbose_name="Including IP")),
                ("relay", models.IntegerField(verbose_name="Relay")),
                (
                    "return_packages",
                    models.IntegerField(verbose_name="Return Packages"),
                ),
                ("return_points", models.IntegerField(verbose_name="Return Points")),
                ("overdue", models.IntegerField(verbose_name="Overdue")),
                ("anomalies", models.IntegerField(verbose_name="Anomalies")),
                ("total_points", models.IntegerField(verbose_name="Total Points")),
                ("full_km", models.IntegerField(verbose_name="Full KM")),
                (
                    "linked_user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Deliveryman",
                    ),
                ),
            ],
            options={
                "verbose_name": "Chronopost Delivery",
                "verbose_name_plural": "Chronopost Delivery",
            },
        ),
    ]



"""
    Helper to publish this Project to PyPi
"""

from pathlib import Path

from poetry_publish.publish import poetry_publish
from poetry_publish.utils.subprocess_utils import verbose_check_call

import xnbtd


PACKAGE_ROOT = Path(xnbtd.__file__).parent.parent


def publish():
    """
    Publish to PyPi
    Call this via:
        $ poetry run publish
    """
    verbose_check_call('make', 'test')  # don't publish if tests fail
    verbose_check_call('make', 'lint')  # don't publish if code style wrong

    poetry_publish(package_root=PACKAGE_ROOT, version=xnbtd.__version__)



from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy as _


class Event(models.Model):
    date = models.DateField(verbose_name=_('Date'))
    title = models.CharField(max_length=100, verbose_name=_('Title'))

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = _('Event')
        verbose_name_plural = _('Events')
        ordering = ['date']


class Rest(models.Model):
    status = models.BooleanField(verbose_name=_('Status'), default=False)
    linked_user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_('Deliveryman'))
    start_date = models.DateField(verbose_name=_('Start Date'))
    end_date = models.DateField(verbose_name=_('End Date'))

    def __str__(self):
        return self.start_date.strftime("%d/%m/%Y") + " - " + self.linked_user.username

    class Meta:
        verbose_name = _('Rest')
        verbose_name_plural = _('Rests')
        ordering = ['start_date']



from datetime import timedelta

from django import template
from django.db.models import Sum


register = template.Library()


@register.simple_tag
def calculate_total(queryset, column):
    """
    Calculate the total for a given column in a queryset

    Arguments:
        queryset (QuerySet): The queryset that is being used to calculate the total.
        column (str): The column for which the total is being calculated.

    Returns:
        float or None: The total of the column rounded to two decimal places,
        or None if the total is None.
    """
    if queryset.exists():
        model_class = queryset.model
        field_type = model_class._meta.get_field(column).get_internal_type()

        if field_type in ("IntegerField", "FloatField"):
            total = queryset.aggregate(total=Sum(column)).get("total")
            return round(total, 2) if total is not None else None
        elif field_type == "TimeField":
            total_time = timedelta()

            for item in queryset:
                if value := getattr(item, column):
                    total_time += timedelta(
                        hours=value.hour, minutes=value.minute, seconds=value.second
                    )

            # Calculate total hours and round to 2 decimal places
            total_hours = total_time.total_seconds() / 3600
            return round(total_hours, 2) if total_hours is not None else None

    return None



# Generated by Django 4.2.6 on 2023-10-24 14:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("tours", "0003_remove_chronopostdelivery_total_hour_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="chronopostdelivery",
            name="breaks",
        ),
        migrations.RemoveField(
            model_name="chronopostpickup",
            name="breaks",
        ),
        migrations.RemoveField(
            model_name="ciblex",
            name="breaks",
        ),
        migrations.RemoveField(
            model_name="gls",
            name="breaks",
        ),
        migrations.RemoveField(
            model_name="tnt",
            name="breaks",
        ),
        migrations.CreateModel(
            name="BreakTime",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("object_id", models.PositiveIntegerField()),
                ("time", models.TimeField(verbose_name="Pause Time")),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
    ]



from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils import formats
from django.utils.translation import gettext_lazy as _


class BaseModel(models.Model):
    linked_user = models.ForeignKey(User, on_delete=models.CASCADE, verbose_name=_("Deliveryman"))
    name = models.CharField(max_length=255, verbose_name=_("Tour Name"))
    date = models.DateField(verbose_name=_("Tour Date"))
    beginning_hour = models.TimeField(verbose_name=_("Beginning Hour"))
    ending_hour = models.TimeField(verbose_name=_("Ending Hour"))
    license_plate = models.CharField(max_length=7, verbose_name=_("License Plate"))
    comments = models.TextField(verbose_name=_("Comments"), null=True, blank=True)

    def __str__(self):
        formatted_date = formats.date_format(self.date, format="l j F Y")
        return f"{self.name} - {formatted_date}"

    class Meta:
        abstract = True


class GLS(BaseModel):
    points_charges = models.IntegerField(verbose_name=_("Points Charges"))
    points_delivered = models.IntegerField(verbose_name=_("Points Delivered"))
    packages_charges = models.IntegerField(verbose_name=_("Packages Charges"))
    packages_delivered = models.IntegerField(verbose_name=_("Packages Delivered"))
    avp_relay = models.IntegerField(verbose_name=_("AVP Relay"))
    shd = models.IntegerField(verbose_name=_("SHD"))
    eo = models.IntegerField(verbose_name=_("EO"))
    pickup_point = models.IntegerField(verbose_name=_("Pickup Point"))
    full_km = models.IntegerField(verbose_name=_("Full KM"))

    class Meta:
        verbose_name = _("GLS")
        verbose_name_plural = _("GLS")


class ChronopostDelivery(BaseModel):
    charged_packages = models.IntegerField(verbose_name=_("Charged Packages"))
    charged_points = models.IntegerField(verbose_name=_("Charged Points"))
    including_ip = models.IntegerField(verbose_name=_("Including IP"))
    relay = models.IntegerField(verbose_name=_("Relay"))
    return_packages = models.IntegerField(verbose_name=_("Return Packages"))
    return_points = models.IntegerField(verbose_name=_("Return Points"))
    overdue = models.IntegerField(verbose_name=_("Overdue"))
    anomalies = models.IntegerField(verbose_name=_("Anomalies"))
    total_points = models.IntegerField(verbose_name=_("Total Points"))
    full_km = models.IntegerField(verbose_name=_("Full KM"))

    class Meta:
        verbose_name = _("Chronopost Delivery")
        verbose_name_plural = _("Chronopost Delivery")


class ChronopostPickup(BaseModel):
    esd = models.IntegerField(verbose_name=_("ESD"))
    picked_points = models.IntegerField(verbose_name=_("Picked Points"))
    poste = models.IntegerField(verbose_name=_("Poste"))

    class Meta:
        verbose_name = _("Chronopost Pickup")
        verbose_name_plural = _("Chronopost Pickup")


class TNT(BaseModel):
    client_numbers = models.IntegerField(verbose_name=_("Client numbers"))
    refused = models.IntegerField(verbose_name=_("Refused"))
    avp = models.IntegerField(verbose_name=_("avp"))
    cad = models.IntegerField(verbose_name=_("cad/return"))
    totals_clients = models.IntegerField(verbose_name=_("totals clients"))
    occasional_abductions = models.IntegerField(verbose_name=_("occasional abductions"))
    regular_abductions = models.IntegerField(verbose_name=_("regular abductions"))
    totals_clients_abductions = models.IntegerField(verbose_name=_("total clients abductions"))
    kilometers = models.IntegerField(verbose_name=_("KM/Full"))

    class Meta:
        verbose_name = _("TNT")
        verbose_name_plural = _("TNT")


class Ciblex(BaseModel):
    type = models.CharField(max_length=255, default="LIVRAISONS")
    nights = models.IntegerField(verbose_name=_("nights"))
    days = models.IntegerField(verbose_name=_("days"))
    avp = models.IntegerField(verbose_name=_("avp"))
    spare_part = models.IntegerField(verbose_name=_("spare part"))
    synchro = models.IntegerField(verbose_name=_("synchro"))
    morning_pickup = models.IntegerField(verbose_name=_("morning pickups"))

    class Meta:
        verbose_name = _("Ciblex")
        verbose_name_plural = _("Ciblex")


class BreakTime(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey("content_type", "object_id")

    start_time = models.TimeField(verbose_name=_("Start of Break"))
    end_time = models.TimeField(verbose_name=_("End of Break"))

    def __str__(self):
        return str(_("Break from {} to {}")).format(self.start_time, self.end_time)

    class Meta:
        verbose_name = _("Break Time")
        verbose_name_plural = _("Break Times")



from django.contrib import admin


class AppAdminSite(admin.AdminSite):
    site_header = "NBTD Transport"
    site_title = "Portail - NBTD Transport"
    index_title = "Bienvenue sur le portail des services de NBTD Transport"
    index_template = "xnbtd/admin/index.html"



from datetime import datetime, timedelta

from django import template

from xnbtd.plannings.models import Event


register = template.Library()


@register.simple_tag
def get_upcoming_events_grouped(count):
    yesterday = datetime.now() - timedelta(days=1)
    events = Event.objects.filter(date__gt=yesterday)[:count]

    grouped_events = {}
    for event in events:
        event_date = event.date
        if event_date == datetime.now().date():
            date_key = "Aujourd'hui"
        elif event_date == (datetime.now() + timedelta(days=1)).date():
            date_key = "Demain"
        else:
            date_key = event_date

        if date_key in grouped_events:
            grouped_events[date_key].append(event)
        else:
            grouped_events[date_key] = [event]

    return grouped_events



# Generated by Django 4.1.10 on 2023-08-18 17:10

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("plannings", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Rest",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("status", models.BooleanField(default=False, verbose_name="Status")),
                ("start_date", models.DateField(verbose_name="Start Date")),
                ("end_date", models.DateField(verbose_name="End Date")),
                (
                    "linked_user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Deliveryman",
                    ),
                ),
            ],
            options={
                "verbose_name": "Rest",
                "verbose_name_plural": "Rests",
                "ordering": ["start_date"],
            },
        ),
    ]



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _

from .forms import RestAdminForm
from .models import Event, Rest


# Filters
class StatusFilter(admin.SimpleListFilter):
    title = _('status')  # Human-readable title for the filter
    parameter_name = 'status'  # URL query parameter

    def lookups(self, request, model_admin):
        # Display values for the filter
        return [
            ('validated', _('validated')),
            ('pending', _('pending')),
        ]

    def queryset(self, request, queryset):
        # Modify the queryset based on the filter value
        if self.value() == 'validated':
            return queryset.filter(status=True)
        elif self.value() == 'pending':
            return queryset.filter(status=False)


# Admins
class EventAdmin(admin.ModelAdmin):
    list_filter = ("date",)

    def changelist_view(self, request, extra_context=None):
        self.date_hierarchy = "date"
        self.list_display = ("title", "date")
        return super().changelist_view(request, extra_context)


class RestAdmin(admin.ModelAdmin):
    form = RestAdminForm
    list_display = ("display_status", "linked_user", "start_date", "end_date")
    list_filter = (StatusFilter, "linked_user")

    def display_status(self, obj):
        if obj.status:
            return format_html('<span style="color: green;">{}</span>', _("validated"))
        return format_html('<span style="color: red;">{}</span>', _("pending"))

    display_status.admin_order_field = "status"
    display_status.short_description = _("Status")

    def get_form(self, request, obj=None, **kwargs):
        form = super(RestAdmin, self).get_form(request, obj, **kwargs)
        form.current_user = request.user
        return form

    def get_queryset(self, request):
        qs = super(RestAdmin, self).get_queryset(request)
        return qs if request.user.is_superuser else qs.filter(linked_user=request.user)

    def get_changeform_initial_data(self, request):
        if not request.user.is_superuser:
            get_data = super(RestAdmin, self).get_changeform_initial_data(request)
            get_data["linked_user"] = request.user.pk
            return get_data
        return super(RestAdmin, self).get_changeform_initial_data(request)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if not request.user.is_superuser:
            if db_field.name == "linked_user":
                kwargs["queryset"] = get_user_model().objects.filter(username=request.user.username)
            return super().formfield_for_foreignkey(db_field, request, **kwargs)
        return super(RestAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


admin.site.register(Event, EventAdmin)
admin.site.register(Rest, RestAdmin)



# Generated by Django 4.1.10 on 2023-09-19 09:46

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("tours", "0002_chronopostdelivery_comments_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="chronopostdelivery",
            name="total_hour",
        ),
        migrations.RemoveField(
            model_name="chronopostpickup",
            name="total_hour",
        ),
        migrations.RemoveField(
            model_name="ciblex",
            name="total_hour",
        ),
        migrations.RemoveField(
            model_name="gls",
            name="total_hour",
        ),
        migrations.RemoveField(
            model_name="tnt",
            name="total_hour",
        ),
    ]



# Generated by Django 4.1.10 on 2023-08-18 16:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("tours", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="chronopostdelivery",
            name="comments",
            field=models.TextField(blank=True, null=True, verbose_name="Comments"),
        ),
        migrations.AddField(
            model_name="chronopostdelivery",
            name="license_plate",
            field=models.CharField(default="XX000XX", max_length=7, verbose_name="License Plate"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="chronopostpickup",
            name="comments",
            field=models.TextField(blank=True, null=True, verbose_name="Comments"),
        ),
        migrations.AddField(
            model_name="chronopostpickup",
            name="license_plate",
            field=models.CharField(default="XX000XX", max_length=7, verbose_name="License Plate"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="ciblex",
            name="comments",
            field=models.TextField(blank=True, null=True, verbose_name="Comments"),
        ),
        migrations.AddField(
            model_name="ciblex",
            name="license_plate",
            field=models.CharField(default="XX000XX", max_length=7, verbose_name="License Plate"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="gls",
            name="comments",
            field=models.TextField(blank=True, null=True, verbose_name="Comments"),
        ),
        migrations.AddField(
            model_name="gls",
            name="full_km",
            field=models.IntegerField(default=0, verbose_name="Full KM"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="gls",
            name="license_plate",
            field=models.CharField(default="XX000XX", max_length=7, verbose_name="License Plate"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="tnt",
            name="comments",
            field=models.TextField(blank=True, null=True, verbose_name="Comments"),
        ),
        migrations.AddField(
            model_name="tnt",
            name="license_plate",
            field=models.CharField(default="XX000XX", max_length=7, verbose_name="License Plate"),
            preserve_default=False,
        ),
    ]



# Generated by Django 4.1.10 on 2023-07-21 21:35

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(verbose_name="Date")),
                ("title", models.CharField(max_length=100, verbose_name="Title")),
            ],
            options={
                "verbose_name": "Event",
                "verbose_name_plural": "Events",
                "ordering": ["date"],
            },
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-24 15:37

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("tours", "0004_remove_chronopostdelivery_breaks_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="breaktime",
            options={
                "verbose_name": "Break Time",
                "verbose_name_plural": "Break Times",
            },
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-26 05:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("tours", "0005_alter_breaktime_options"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="breaktime",
            name="time",
        ),
        migrations.AddField(
            model_name="breaktime",
            name="end_time",
            field=models.TimeField(default="00:00", verbose_name="End of Break"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="breaktime",
            name="start_time",
            field=models.TimeField(default="00:00", verbose_name="Start of Break"),
            preserve_default=False,
        ),
    ]
